@@ -1,55 +1,56 @@
 
 #	pre-5.14.0 perl inadvertently destroys signal handlers
 #	http://rt.perl.org/rt3/Public/Bug/Display.html?id=76138
-#
-BEGIN {					## capture %SIG before compilation
-	use constant RT_76138 => $] < 5.014;
-	@::SIG_BACKUP = %SIG if RT_76138;
-}
-
-sub UNITCHECK {				## restore %SIG after compilation
-	%SIG = @::SIG_BACKUP if RT_76138;
-}
+use strict;
+use warnings;
+local %SIG = %SIG;
 
 
 package Net::DNS::RR::SIG;
 
-#
-# $Id: SIG.pm 1561 2017-04-19 13:08:13Z willem $
-#
-our $VERSION = (qw$LastChangedRevision: 1561 $)[1];
-
-
 use strict;
 use warnings;
+our $VERSION = (qw$Id: SIG.pm 1856 2021-12-02 14:36:25Z willem $)[2];
+
 use base qw(Net::DNS::RR);
 
+
 =head1 NAME
 
 Net::DNS::RR::SIG - DNS SIG resource record
 
 =cut
 
-
 use integer;
 
 use Carp;
 use Time::Local;
 
-eval 'require MIME::Base64';
-
-use Net::DNS::Parameters;
+use Net::DNS::Parameters qw(:type);
 
 use constant DEBUG => 0;
 
-use constant UTIL => defined eval 'use Scalar::Util 1.25; 1;';
+use constant UTIL => defined eval { require Scalar::Util; };
 
-use constant PRIVATE => defined eval 'require Net::DNS::SEC::Private';
+eval { require MIME::Base64 };
 
-use constant DSA => defined eval 'require Net::DNS::SEC::DSA';
-use constant RSA => defined eval 'require Net::DNS::SEC::RSA';
+# IMPORTANT: Downstream distros MUST NOT create dependencies on Net::DNS::SEC	(strong crypto prohibited in many territories)
+use constant USESEC => defined $INC{'Net/DNS/SEC.pm'};		# Discover how we got here, without exposing any crypto
+use constant							# Discourage static code analysers and casual greppers
+		DNSSEC => USESEC && defined eval join '',
+		qw(r e q u i r e), ' Net::DNS', qw(:: SEC :: Private);	  ## no critic
 
-use constant DNSSEC => PRIVATE && ( RSA || DSA );
+my @index;
+if (DNSSEC) {
+	foreach my $class ( map {"Net::DNS::SEC::$_"} qw(RSA DSA ECCGOST ECDSA EdDSA) ) {
+		my @algorithms = eval join '', qw(r e q u i r e), " $class; $class->_index";	## no critic
+		push @index, map { ( $_ => $class ) } @algorithms;
+	}
+	croak 'Net::DNS::SEC version not supported' unless scalar(@index);
+}
+
+my %DNSSEC_verify = @index;
+my %DNSSEC_siggen = @index;
 
 my @field = qw(typecovered algorithm labels orgttl sigexpiration siginception keytag);
 
@@ -60,12 +61,13 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 
 	my $limit = $offset + $self->{rdlength};
 	@{$self}{@field} = unpack "\@$offset n C2 N3 n", $$data;
-	( $self->{signame}, $offset ) = decode Net::DNS::DomainName2535( $data, $offset + 18 );
+	( $self->{signame}, $offset ) = Net::DNS::DomainName2535->decode( $data, $offset + 18 );
 	$self->{sigbin} = substr $$data, $offset, $limit - $offset;
 
 	croak('misplaced or corrupt SIG') unless $limit == length $$data;
 	my $raw = substr $$data, 0, $self->{offset};
 	$self->{rawref} = \$raw;
+	return;
 }
 
 
@@ -75,26 +77,25 @@ sub _encode_rdata {			## encode rdata as wire-format octet string
 
 	my ( $hash, $packet ) = @opaque;
 
-	my $signame = $self->{signame} || return '';
+	my $signame = $self->{signame};
 
 	if ( DNSSEC && !$self->{sigbin} ) {
-		my $private = $self->{private} || die 'missing key reference';
-		delete $self->{private};			# one shot is all you get
-
+		my $private = delete $self->{private};		# one shot is all you get
 		my $sigdata = $self->_CreateSigData($packet);
-		$self->_CreateSig( $sigdata, $private );
+		$self->_CreateSig( $sigdata, $private || die 'missing key reference' );
 	}
 
-	pack 'n C2 N3 n a* a*', @{$self}{@field}, $signame->encode, $self->sigbin;
+	return pack 'n C2 N3 n a* a*', @{$self}{@field}, $signame->encode, $self->sigbin;
 }
 
 
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	my $signame = $self->{signame} || return '';
+	my $sname = $self->{signame} || return '';
 	my @sig64 = split /\s+/, MIME::Base64::encode( $self->sigbin );
-	my @rdata = ( map( $self->$_, @field ), $signame->string, @sig64 );
+	my @rdata = ( map( { $self->$_ } @field ), $sname->string, @sig64 );
+	return @rdata;
 }
 
 
@@ -103,6 +104,7 @@ sub _parse_rdata {			## populate RR from rdata in argument list
 
 	foreach ( @field, qw(signame) ) { $self->$_(shift) }
 	$self->signature(@_);
+	return;
 }
 
 
@@ -115,92 +117,7 @@ sub _defaults {				## specify RR attribute default values
 	$self->labels(0);
 	$self->orgttl(0);
 	$self->sigval(10);
-}
-
-
-#
-# source: http://www.iana.org/assignments/dns-sec-alg-numbers
-#
-{
-	my @algbyname = (
-		'DELETE'	     => 0,			# [RFC4034][RFC4398][RFC8087]
-		'RSAMD5'	     => 1,			# [RFC3110][RFC4034]
-		'DH'		     => 2,			# [RFC2539]
-		'DSA'		     => 3,			# [RFC3755][RFC2536]
-					## Reserved	=> 4,	# [RFC6725]
-		'RSASHA1'	     => 5,			# [RFC3110][RFC4034]
-		'DSA-NSEC3-SHA1'     => 6,			# [RFC5155]
-		'RSASHA1-NSEC3-SHA1' => 7,			# [RFC5155]
-		'RSASHA256'	     => 8,			# [RFC5702]
-					## Reserved	=> 9,	# [RFC6725]
-		'RSASHA512'	     => 10,			# [RFC5702]
-					## Reserved	=> 11,	# [RFC6725]
-		'ECC-GOST'	     => 12,			# [RFC5933]
-		'ECDSAP256SHA256'    => 13,			# [RFC6605]
-		'ECDSAP384SHA384'    => 14,			# [RFC6605]
-		'ED25519'	     => 15,			# [RFC8080]
-		'ED448'		     => 16,			# [RFC8080]
-
-		'INDIRECT'   => 252,				# [RFC4034]
-		'PRIVATEDNS' => 253,				# [RFC4034]
-		'PRIVATEOID' => 254,				# [RFC4034]
-					## Reserved	=> 255,	# [RFC4034]
-		);
-
-	my %algbyval = reverse @algbyname;
-
-	my $map = sub {
-		my $arg = shift;
-		unless ( $arg =~ /^\d/ ) {
-			$arg =~ s/[^A-Za-z0-9]//g;		# synthetic key
-			return uc $arg;
-		}
-		my @map = ( $arg, "$arg" => $arg );		# also accept number
-	};
-
-	my %algbyname = map &$map($_), @algbyname;
-
-	sub _algbyname {
-		my $arg = shift;
-		my $key = uc $arg;				# synthetic key
-		$key =~ s/[^A-Z0-9]//g;				# strip non-alphanumerics
-		my $val = $algbyname{$key};
-		return $val if defined $val;
-		return $key =~ /^\d/ ? $arg : croak "unknown algorithm $arg";
-	}
-
-	sub _algbyval {
-		my $value = shift;
-		$algbyval{$value} || return $value;
-	}
-}
-
-
-my $DSA = DSA ? 'Net::DNS::SEC::DSA' : 0;
-my $RSA = RSA ? 'Net::DNS::SEC::RSA' : 0;
-
-my %SEC = (
-	1 => $RSA,
-	3 => $DSA,
-	5 => $RSA,
-	6 => $DSA,
-	7 => $RSA,
-	);
-
-my %siglen = (
-	1 => 128,
-	3 => 41,
-	5 => 256,
-	6 => 41,
-	7 => 256,
-	);
-
-
-sub _size {				## estimate encoded size
-	my $self = shift;
-	my $clone = bless {%$self}, ref($self);			   # shallow clone
-	$clone->sigbin( 'x' x $siglen{$self->algorithm} );
-	length $clone->encode();
+	return;
 }
 
 
@@ -208,7 +125,7 @@ sub typecovered {
 	my $self = shift;					# uncoverable pod
 	$self->{typecovered} = typebyname(shift) if scalar @_;
 	my $typecode = $self->{typecovered};
-	typebyval($typecode) if defined wantarray && defined $typecode;
+	return defined $typecode ? typebyval($typecode) : undef;
 }
 
 
@@ -227,12 +144,12 @@ sub algorithm {
 
 
 sub labels {
-	shift->{labels} = 0;					# uncoverable pod
+	return shift->{labels} = 0;				# uncoverable pod
 }
 
 
 sub orgttl {
-	shift->{orgttl} = 0;					# uncoverable pod
+	return shift->{orgttl} = 0;				# uncoverable pod
 }
 
 
@@ -252,14 +169,15 @@ sub siginception {
 	return UTIL ? Scalar::Util::dualvar( $time, _time2string($time) ) : _time2string($time);
 }
 
-sub sigex { &sigexpiration; }		## historical
+sub sigex { return &sigexpiration; }	## historical
 
-sub sigin { &siginception; }		## historical
+sub sigin { return &siginception; }	## historical
 
 sub sigval {
 	my $self = shift;
 	no integer;
-	( $self->{sigval} ) = map int( 60.0 * $_ ), @_;
+	( $self->{sigval} ) = map { int( 60.0 * $_ ) } @_;
+	return;
 }
 
 
@@ -267,23 +185,22 @@ sub keytag {
 	my $self = shift;
 
 	$self->{keytag} = 0 + shift if scalar @_;
-	$self->{keytag} || 0;
+	return $self->{keytag} || 0;
 }
 
 
 sub signame {
 	my $self = shift;
 
-	$self->{signame} = new Net::DNS::DomainName2535(shift) if scalar @_;
-	$self->{signame}->name if $self->{signame};
+	$self->{signame} = Net::DNS::DomainName2535->new(shift) if scalar @_;
+	return $self->{signame} ? $self->{signame}->name : undef;
 }
 
 
 sub sig {
 	my $self = shift;
-
-	$self->sigbin( MIME::Base64::decode( join "", @_ ) ) if scalar @_;
-	MIME::Base64::encode( $self->sigbin(), "" ) if defined wantarray;
+	return MIME::Base64::encode( $self->sigbin(), "" ) unless scalar @_;
+	return $self->sigbin( MIME::Base64::decode( join "", @_ ) );
 }
 
 
@@ -291,23 +208,23 @@ sub sigbin {
 	my $self = shift;
 
 	$self->{sigbin} = shift if scalar @_;
-	$self->{sigbin} || "";
+	return $self->{sigbin} || "";
 }
 
 
-sub signature { &sig; }
+sub signature { return &sig; }
 
 
 sub create {
 	unless (DNSSEC) {
-		croak 'Net::DNS::SEC support not available';
+		croak qq[No "use Net::DNS::SEC" declaration in application code];
 	} else {
 		my ( $class, $data, $priv_key, %etc ) = @_;
 
-		my $private = ref($priv_key) ? $priv_key : Net::DNS::SEC::Private->new($priv_key);
+		my $private = ref($priv_key) ? $priv_key : ( Net::DNS::SEC::Private->new($priv_key) );
 		croak 'Unable to parse private key' unless ref($private) eq 'Net::DNS::SEC::Private';
 
-		my $self = new Net::DNS::RR(
+		my $self = Net::DNS::RR->new(
 			type	     => 'SIG',
 			typecovered  => 'TYPE0',
 			siginception => time(),
@@ -341,34 +258,33 @@ sub verify {
 	# $keyref is either a key object or a reference to an array
 	# of keys.
 
-	if (DNSSEC) {
+	unless (DNSSEC) {
+		croak qq[No "use Net::DNS::SEC" declaration in application code];
+	} else {
 		my ( $self, $dataref, $keyref ) = @_;
 
 		if ( my $isa = ref($dataref) ) {
 			print '$dataref argument is ', $isa, "\n" if DEBUG;
-			croak '$dataref can not be ', $isa unless $isa =~ /^Net::DNS::/;
-			croak '$dataref can not be ', $isa unless $dataref->isa('Net::DNS::Packet');
+			croak '$dataref must be scalar or a Net::DNS::Packet'
+					unless $isa =~ /Net::DNS/ && $dataref->isa('Net::DNS::Packet');
 		}
 
 		print '$keyref argument is of class ', ref($keyref), "\n" if DEBUG;
 		if ( ref($keyref) eq "ARRAY" ) {
 
-			#  We will recurse for each key that matches algorithm and key-id
-			#  and return when there is a successful verification.
-			#  If not, we'll continue so that we even survive key-id collision.
-			#  The downside of this is that the error string only matches the
-			#  last error.
+			#  We will iterate over the supplied key list and
+			#  return when there is a successful verification.
+			#  If not, continue so that we survive key-id collision.
 
 			print "Iterating over ", scalar(@$keyref), " keys\n" if DEBUG;
 			my @error;
-			my $i;
 			foreach my $keyrr (@$keyref) {
 				my $result = $self->verify( $dataref, $keyrr );
 				return $result if $result;
 				my $error = $self->{vrfyerrstr};
-				$i++;
-				push @error, "key $i: $error";
-				print "key $i: $error\n" if DEBUG;
+				my $keyid = $keyrr->keytag;
+				push @error, "key $keyid: $error";
+				print "key $keyid: $error\n" if DEBUG;
 				next;
 			}
 
@@ -383,6 +299,7 @@ sub verify {
 			croak join ' ', ref($keyref), 'can not be used as SIG0 key';
 		}
 
+		croak "SIG typecovered is TYPE$self->{typecovered}" if $self->{typecovered};
 
 		if (DEBUG) {
 			print "\n ---------------------- SIG DEBUG ----------------------";
@@ -391,8 +308,6 @@ sub verify {
 			print "\n -------------------------------------------------------\n";
 		}
 
-		croak "Trying to verify SIG0 using non-SIG0 signature" if $self->{typecovered};
-
 		$self->{vrfyerrstr} = '';
 		unless ( $self->algorithm == $keyref->algorithm ) {
 			$self->{vrfyerrstr} = 'algorithm does not match';
@@ -426,26 +341,165 @@ sub verify {
 
 
 sub vrfyerrstr {
-	shift->{vrfyerrstr};
+	return shift->{vrfyerrstr};
 }
 
 
 ########################################
 
-sub _ordered($$) {			## irreflexive 32-bit partial ordering
-	use integer;
-	my ( $a, $b ) = @_;
+{
+	my @algbyname = (
+		'DELETE'	     => 0,			# [RFC4034][RFC4398][RFC8078]
+		'RSAMD5'	     => 1,			# [RFC3110][RFC4034]
+		'DH'		     => 2,			# [RFC2539]
+		'DSA'		     => 3,			# [RFC3755][RFC2536]
+					## Reserved	=> 4,	# [RFC6725]
+		'RSASHA1'	     => 5,			# [RFC3110][RFC4034]
+		'DSA-NSEC3-SHA1'     => 6,			# [RFC5155]
+		'RSASHA1-NSEC3-SHA1' => 7,			# [RFC5155]
+		'RSASHA256'	     => 8,			# [RFC5702]
+					## Reserved	=> 9,	# [RFC6725]
+		'RSASHA512'	     => 10,			# [RFC5702]
+					## Reserved	=> 11,	# [RFC6725]
+		'ECC-GOST'	     => 12,			# [RFC5933]
+		'ECDSAP256SHA256'    => 13,			# [RFC6605]
+		'ECDSAP384SHA384'    => 14,			# [RFC6605]
+		'ED25519'	     => 15,			# [RFC8080]
+		'ED448'		     => 16,			# [RFC8080]
 
-	return defined $b unless defined $a;			# ( undef, any )
-	return 0 unless defined $b;				# ( any, undef )
+		'INDIRECT'   => 252,				# [RFC4034]
+		'PRIVATEDNS' => 253,				# [RFC4034]
+		'PRIVATEOID' => 254,				# [RFC4034]
+					## Reserved	=> 255,	# [RFC4034]
+		);
 
-	# unwise to assume 64-bit arithmetic, or that 32-bit integer overflow goes unpunished
-	if ( $a < 0 ) {						# translate $a<0 region
-		$a = ( $a ^ 0x80000000 ) & 0xFFFFFFFF;		#  0	 <= $a < 2**31
-		$b = ( $b ^ 0x80000000 ) & 0xFFFFFFFF;		# -2**31 <= $b < 2**32
+	my %algbyval = reverse @algbyname;
+
+	foreach (@algbyname) { s/[\W_]//g; }			# strip non-alphanumerics
+	my @algrehash = map { /^\d/ ? ($_) x 3 : uc($_) } @algbyname;
+	my %algbyname = @algrehash;				# work around broken cperl
+
+	sub _algbyname {
+		my $arg = shift;
+		my $key = uc $arg;				# synthetic key
+		$key =~ s/[\W_]//g;				# strip non-alphanumerics
+		my $val = $algbyname{$key};
+		return $val if defined $val;
+		return $key =~ /^\d/ ? $arg : croak qq[unknown algorithm $arg];
 	}
 
-	return $a < $b ? ( $a > ( $b - 0x80000000 ) ) : ( $b < ( $a - 0x80000000 ) );
+	sub _algbyval {
+		my $value = shift;
+		return $algbyval{$value} || return $value;
+	}
+}
+
+
+{
+	my %siglen = (
+		1  => 128,
+		3  => 41,
+		5  => 256,
+		6  => 41,
+		7  => 256,
+		8  => 256,
+		10 => 256,
+		12 => 64,
+		13 => 64,
+		14 => 96,
+		15 => 64,
+		16 => 114,
+		);
+
+	sub _size {			## estimate encoded size
+		my $self  = shift;
+		my $clone = bless {%$self}, ref($self);		# shallow clone
+		$clone->sigbin( 'x' x $siglen{$self->algorithm} );
+		return length $clone->encode();
+	}
+}
+
+
+sub _CreateSigData {
+	if (DNSSEC) {
+		my ( $self, $message ) = @_;
+
+		if ( ref($message) ) {
+			die 'missing packet reference' unless $message->isa('Net::DNS::Packet');
+			my @unsigned = grep { ref($_) ne ref($self) } @{$message->{additional}};
+			local $message->{additional} = \@unsigned;    # remake header image
+			my @part = qw(question answer authority additional);
+			my @size = map { scalar @{$message->{$_}} } @part;
+			my $rref = delete $self->{rawref};
+			my $data = $rref ? $$rref : $message->data;
+			my ( $id, $status ) = unpack 'n2', $data;
+			my $hbin = pack 'n6 a*', $id, $status, @size;
+			$message = $hbin . substr $data, length $hbin;
+		}
+
+		my $sigdata = pack 'n C2 N3 n a*', @{$self}{@field}, $self->{signame}->encode;
+		print "\npreamble\t", unpack( 'H*', $sigdata ), "\nrawdata\t", unpack( 'H100', $message ), " ...\n"
+				if DEBUG;
+		return join '', $sigdata, $message;
+	}
+}
+
+
+sub _CreateSig {
+	if (DNSSEC) {
+		my $self = shift;
+
+		my $algorithm = $self->algorithm;
+		my $class     = $DNSSEC_siggen{$algorithm};
+
+		return eval {
+			die "algorithm $algorithm not supported\n" unless $class;
+			$self->sigbin( $class->sign(@_) );
+		} || return croak "${@}signature generation failed";
+	}
+}
+
+
+sub _VerifySig {
+	if (DNSSEC) {
+		my $self = shift;
+
+		my $algorithm = $self->algorithm;
+		my $class     = $DNSSEC_verify{$algorithm};
+
+		my $retval = eval {
+			die "algorithm $algorithm not supported\n" unless $class;
+			$class->verify( @_, $self->sigbin );
+		};
+
+		unless ($retval) {
+			$self->{vrfyerrstr} = "${@}signature verification failed";
+			print "\n", $self->{vrfyerrstr}, "\n" if DEBUG;
+			return 0;
+		}
+
+		# uncoverable branch true	# bug in Net::DNS::SEC or dependencies
+		croak "unknown error in $class->verify" unless $retval == 1;
+		print "\nalgorithm $algorithm verification successful\n" if DEBUG;
+		return 1;
+	}
+}
+
+
+sub _ordered() {			## irreflexive 32-bit partial ordering
+	use integer;
+	my ( $n1, $n2 ) = @_;
+
+	return 0 unless defined $n2;				# ( any, undef )
+	return 1 unless defined $n1;				# ( undef, any )
+
+	# unwise to assume 64-bit arithmetic, or that 32-bit integer overflow goes unpunished
+	if ( $n2 < 0 ) {					# fold, leaving $n2 non-negative
+		$n1 = ( $n1 & 0xFFFFFFFF ) ^ 0x80000000;	# -2**31 <= $n1 < 2**32
+		$n2 = ( $n2 & 0x7FFFFFFF );			#  0	 <= $n2 < 2**31
+	}
+
+	return $n1 < $n2 ? ( $n1 > ( $n2 - 0x80000000 ) ) : ( $n2 < ( $n1 - 0x80000000 ) );
 }
 
 
@@ -460,24 +514,21 @@ my $t2100 = 1960058752;
 
 sub _string2time {			## parse time specification string
 	my $arg = shift;
-	croak 'undefined time' unless defined $arg;
 	return int($arg) if length($arg) < 12;
 	my ( $y, $m, @dhms ) = unpack 'a4 a2 a2 a2 a2 a2', $arg . '00';
 	if ( $arg lt '20380119031408' ) {			# calendar folding
 		return timegm( reverse(@dhms), $m - 1, $y ) if $y < 2026;
 		return timegm( reverse(@dhms), $m - 1, $y - 56 ) + $y2026;
 	} elsif ( $y > 2082 ) {
-		my $z = timegm( reverse(@dhms), $m - 1, $y - 84 );
-		$z -= 86400 unless $z < 1456704000 + 86400;	# expunge 29 Feb 2100
-		return $z + $y2054;
+		my $z = timegm( reverse(@dhms), $m - 1, $y - 84 );    # expunge 29 Feb 2100
+		return $z < 1456790400 ? $z + $y2054 : $z + $y2054 - 86400;
 	}
 	return ( timegm( reverse(@dhms), $m - 1, $y - 56 ) + $y2054 ) - $y1998;
 }
 
 
 sub _time2string {			## format time specification string
-	my $arg = shift;
-	croak 'undefined time' unless defined $arg;
+	my $arg	 = shift;
 	my $ls31 = int( $arg & 0x7FFFFFFF );
 	if ( $arg & 0x80000000 ) {
 
@@ -500,76 +551,8 @@ sub _time2string {			## format time specification string
 	return sprintf '%d%02d%02d%02d%02d%02d', $yy + 1900, $mm + 1, @dhms;
 }
 
-
-sub _CreateSigData {
-	if (DNSSEC) {
-		my ( $self, $message ) = @_;
-
-		if ( ref($message) ) {
-			die 'missing packet reference' unless $message->isa('Net::DNS::Packet');
-			my @unsigned = grep ref($_) ne ref($self), @{$message->{additional}};
-			local $message->{additional} = \@unsigned;    # remake header image
-			my @part = qw(question answer authority additional);
-			my @size = map scalar( @{$message->{$_}} ), @part;
-			my $rref = $self->{rawref};
-			delete $self->{rawref};
-			my $data = $rref ? $$rref : $message->data;
-			my ( $id, $status ) = unpack 'n2', $data;
-			my $hbin = pack 'n6 a*', $id, $status, @size;
-			$message = $hbin . substr $data, length $hbin;
-		}
-
-		my @field = qw(typecovered algorithm labels orgttl sigexpiration siginception keytag);
-		my $sigdata = pack 'n C2 N3 n a*', @{$self}{@field}, $self->{signame}->encode;
-		print "\npreamble\t", unpack( 'H*', $sigdata ), "\nrawdata\t", unpack( 'H100', $message ), " ...\n"
-				if DEBUG;
-		join '', $sigdata, $message;
-	}
-}
-
-
 ########################################
 
-sub _CreateSig {
-	if (DNSSEC) {
-		my $self = shift;
-
-		my $algorithm = $self->algorithm;
-		my $class     = $SEC{$algorithm};
-
-		eval {
-			die "algorithm $algorithm not supported" unless $class;
-			$self->sigbin( $class->sign(@_) );
-		} || croak "${@}signature generation failed";
-	}
-}
-
-
-sub _VerifySig {
-	if (DNSSEC) {
-		my $self = shift;
-
-		my $algorithm = $self->algorithm;
-		my $class     = $SEC{$algorithm};
-
-		my $retval = eval {
-			die "algorithm $algorithm not supported" unless $class;
-			$class->verify( @_, $self->sigbin );
-		};
-
-		unless ($retval) {
-			$self->{vrfyerrstr} = "${@}signature verification failed";
-			print "\n", $self->{vrfyerrstr}, "\n" if DEBUG;
-			return 0;
-		}
-
-		# uncoverable branch true	# bug in Net::DNS::SEC or dependencies
-		croak "unknown error in $class->verify" unless $retval == 1;
-		print "\nalgorithm $algorithm verification successful\n" if DEBUG;
-		return 1;
-	}
-}
-
 
 1;
 __END__
@@ -578,12 +561,12 @@ __END__
 =head1 SYNOPSIS
 
     use Net::DNS;
-    $rr = new Net::DNS::RR('name SIG typecovered algorithm labels
+    $rr = Net::DNS::RR->new('name SIG typecovered algorithm labels
 				orgttl sigexpiration siginception
 				keytag signame signature');
 
     use Net::DNS::SEC;
-    $sigrr = create Net::DNS::RR::SIG( $string, $keypath,
+    $sigrr = Net::DNS::RR::SIG->create( $string, $keypath,
 					sigval => 10	# minutes
 					);
 
@@ -681,9 +664,9 @@ Create a signature over scalar data.
 
     $keypath = '/home/olaf/keys/Kbla.foo.+001+60114.private';
 
-    $sigrr = create Net::DNS::RR::SIG( $data, $keypath );
+    $sigrr = Net::DNS::RR::SIG->create( $data, $keypath );
 
-    $sigrr = create Net::DNS::RR::SIG( $data, $keypath,
+    $sigrr = Net::DNS::RR::SIG->create( $data, $keypath,
 					sigval => 10
 					);
     $sigrr->print;
@@ -693,7 +676,7 @@ Create a signature over scalar data.
 
     $private = Net::DNS::SEC::Private->new($keypath);
 
-    $sigrr= create Net::DNS::RR::SIG( $data, $private );
+    $sigrr= Net::DNS::RR::SIG->create( $data, $private );
 
 
 create() is an alternative constructor for a SIG RR object.  
@@ -704,14 +687,14 @@ the private key stored in the key file.
 The first argument is a scalar that contains the data to be signed.
 
 The second argument is a string which specifies the path to a file
-containing the private key as generated with dnssec-keygen, a program
+containing the private key as generated using dnssec-keygen, a program
 that comes with the ISC BIND distribution.
 
 The optional remaining arguments consist of ( name => value ) pairs
 as follows:
 
-	sigin  => 20171201010101,	# signature inception
-	sigex  => 20171201011101,	# signature expiration
+	sigin  => 20211201010101,	# signature inception
+	sigex  => 20211201011101,	# signature expiration
 	sigval => 10,			# validity window (minutes)
 
 The sigin and sigex values may be specified as Perl time values or as
@@ -727,9 +710,9 @@ By default the signature is valid for 10 minutes.
 
 =item *
 
-Do not change the name of the file generated by dnssec-keygen, the
-create method uses the filename as generated by dnssec-keygen to
-determine the keyowner, algorithm and the keyid (keytag).
+Do not change the name of the private key file.
+The create method uses the filename as generated by dnssec-keygen
+to determine the keyowner, algorithm, and the keyid (keytag).
 
 =back
 
@@ -764,15 +747,18 @@ Returns false on error and sets $sig->vrfyerrstr
 The code is not optimised for speed.
 
 If this code is still around in 2100 (not a leap year) you will
-need to check for proper handling of times ...
+need to check for proper handling of times after 28th February.
 
 =head1 ACKNOWLEDGMENTS
 
-Andy Vaskys (Network Associates Laboratories) supplied the code for
-handling RSA with SHA1 (Algorithm 5).
+Although their original code may have disappeared following redesign of
+Net::DNS, Net::DNS::SEC and the OpenSSL API, the following individual
+contributors deserve to be recognised for their significant influence
+on the development of the SIG package.
 
-T.J. Mather, the Crypt::OpenSSL::DSA maintainer, for his quick
-responses to bug report and feature requests.
+Andy Vaskys (Network Associates Laboratories) supplied code for RSA.
+
+T.J. Mather provided support for the DSA algorithm.
 
 
 =head1 COPYRIGHT
@@ -792,7 +778,7 @@ Package template (c)2009,2012 O.M.Kolkman and R.W.Franks.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
@@ -810,12 +796,10 @@ DEALINGS IN THE SOFTWARE.
 =head1 SEE ALSO
 
 L<perl>, L<Net::DNS>, L<Net::DNS::RR>, L<Net::DNS::SEC>,
-RFC4034, RFC3755, RFC2535, RFC2931, RFC3110, RFC3008,
-L<Net::DNS::SEC::DSA>,
-L<Net::DNS::SEC::RSA>
+RFC2536, RFC2931, RFC3110, RFC4034
 
 L<Algorithm Numbers|http://www.iana.org/assignments/dns-sec-alg-numbers>
 
-L<BIND 9 Administrator Reference Manual|http://www.bind9.net/manuals>
+L<BIND Administrator Reference Manual|http://bind.isc.org/>
 
 =cut
