@@ -2,7 +2,7 @@ package Net::DNS::Resolver::Base;
 
 use strict;
 use warnings;
-our $VERSION = (qw$Id: Base.pm 1855 2021-11-26 11:33:48Z willem $)[2];
+our $VERSION = (qw$Id: Base.pm 1883 2022-11-03 14:38:19Z willem $)[2];
 
 
 #
@@ -257,7 +257,7 @@ END
 
 
 sub print {
-	print &string;
+	print shift->string;
 	return;
 }
 
@@ -1043,6 +1043,8 @@ sub tsig {
 	my $self = shift;
 	$self->{tsig_rr} = eval {
 		local $SIG{__DIE__};
+		return shift unless $_[0];
+		return shift if ref($_[0]) eq 'Net::DNS::RR::TSIG';
 		require Net::DNS::RR::TSIG;
 		Net::DNS::RR::TSIG->create(@_);
 	};
